@@ -1,12 +1,13 @@
 #include "mlir/Dialect/Rock/Generator/ConvGenerator.h"
 #include "mlir/Dialect/AMDGPU/Utils/Chipset.h"
 #include "mlir/Dialect/Func/IR/FuncOps.h"
+#include "mlir/Dialect/Rock/IR/AmdArchDb.h"
 #include "mlir/Dialect/Rock/IR/GemmSize.h"
+#include "mlir/Dialect/Rock/IR/GetRockInfo.h"
 #include "mlir/Dialect/Rock/IR/Rock.h"
 #include "mlir/Dialect/Rock/IR/RockTypes.h"
 #include "mlir/Dialect/Rock/Tuning/ConvContext.h"
 #include "mlir/Dialect/Rock/Tuning/GridwiseGemmParams.h"
-#include "mlir/Dialect/Rock/utility/AmdArchDb.h"
 #include "mlir/Dialect/Rock/utility/builderUtils.h"
 #include "mlir/Dialect/Rock/utility/loweringUtils.h"
 #include "mlir/Dialect/Rock/utility/math.h"
@@ -39,24 +40,24 @@ using namespace mlir::rock;
 
 ConvGenerator::ConvGenerator(
     const std::string &arch, const std::string &chip,
-    bool disableSplitKForTuning, const std::string &triple,
-    const std::string &chipFeatures, const std::string &perfConfig,
-    std::optional<int> num_cu, bool reverseGrid, GemmFeatures features,
-    const std::optional<ConvOpType> operation,
+    bool disableSplitKForTuning, int64_t scheduleVersion,
+    const std::string &triple, const std::string &chipFeatures,
+    const std::string &perfConfig, std::optional<int> num_cu,
+    GemmFeatures features, const std::optional<ConvOpType> operation,
     const std::string &filterDataTypeStr, const std::string &inputDataTypeStr,
     const std::string &outputDataTypeStr, ArrayRef<int> dilations,
     ArrayRef<int> strides, ArrayRef<int> paddingLeft,
     ArrayRef<int> paddingRight, const std::string &filterLayout,
     const std::string &inputLayout, const std::string &outputLayout,
-    const std::string &kernelBaseName)
+    const bool usesV4R1, const std::string &kernelBaseName)
     : config{arch,
              chip,
              disableSplitKForTuning,
+             scheduleVersion,
              triple,
              chipFeatures,
              perfConfig,
              num_cu,
-             reverseGrid,
              features,
              operation,
              filterDataTypeStr,
@@ -69,6 +70,7 @@ ConvGenerator::ConvGenerator(
              filterLayout,
              inputLayout,
              outputLayout,
+             usesV4R1,
              kernelBaseName,
              -1,
              {},
@@ -121,15 +123,11 @@ static LogicalResult hasDimensions(const llvm::StringMap<int64_t> &map,
   return success();
 }
 
-LogicalResult ConvGenerator::isApplicable(bool checkChip) const {
+LogicalResult ConvGenerator::isApplicable() const {
   if (failed(hasValidDimension())) {
     return failure();
   }
 
-  if (checkChip && failed(hasValidChip())) {
-    return failure();
-  }
-
   return success();
 }
 
@@ -247,35 +245,6 @@ LogicalResult ConvGenerator::hasValidDimension() const {
   return success();
 }
 
-LogicalResult ConvGenerator::hasValidChip() const {
-  // We support in between gfx900 to gfx908 and gfx1030 for nonxdlops algorithm
-  // For example, gfx803, gfx90c are unsupported now
-  unsigned int chipHexNumber = 0;
-  if (sscanf(config.chip.c_str(), "gfx%x", &chipHexNumber) != 1)
-    return failure();
-
-  constexpr size_t NUM_SUPPORTED_CHIPS = 12;
-  static const unsigned int supportedChips[NUM_SUPPORTED_CHIPS] = {
-      0x900, 0x906,  0x908,  0x90a,  0x940,  0x941,
-      0x942, 0x1030, 0x1100, 0x1101, 0x1102, 0x1103};
-  const unsigned int *ptr;
-  ptr = std::find(supportedChips, supportedChips + NUM_SUPPORTED_CHIPS,
-                  chipHexNumber);
-  if (ptr == supportedChips + NUM_SUPPORTED_CHIPS)
-    return failure();
-
-  // XDLOPS are only supported on MI-100 (gfx908) and MI-200 (gfx90a)
-  if (bitEnumContainsAll(config.features, GemmFeatures::mfma) &&
-      (chipHexNumber != 0x908 && chipHexNumber != 0x90a))
-    return failure();
-
-  // WMMA is only supported on gfx11xx
-  if (bitEnumContainsAll(config.features, GemmFeatures::wmma) &&
-      (chipHexNumber > 0x1103))
-    return failure();
-  return success();
-}
-
 LogicalResult ConvGenerator::getKernelCount(OpBuilder &builder,
                                             int &kernelCount) const {
   if (config.kernelId > 0) { // generate only 1 specified kernel
@@ -309,28 +278,17 @@ LogicalResult ConvGenerator::getBwdWeightKernelCount(OpBuilder &builder,
     }
     if (!needExtraPad) {
       Type dataType = getInputDataType(builder);
-      if (dataType.isF32()) {
+      if (dataType.isF16()) {
         // For the following case, use 2 kernels:
         // - backward weight
         // - XDLOPS
-        // - fp32
-        // - No need extra pad along Gemm M/N/K
-        // The first kernel will 0-initialize the output (filter tensor).
-        // The second kernel will conduct the actual backward weight
-        // convolution, using atomic add instructions.
-        kernelCount = 2;
-      } else if (dataType.isF16()) {
-        // For the following case, use 3 kernels:
-        // - backward weight
-        // - XDLOPS
         // - fp16
         // - No need extra pad along Gemm M/N/K
-        // The first kernel will 0-initialize the workspace.
-        // The second kernel will conduct the actual backward weight
-        // convolution, using atomic add instructions. The third kernel will do
+        // The first kernel will conduct the actual backward weight
+        // convolution, using atomic add instructions. The second kernel will do
         // elementwise conversion from fp32 in the workspace to fp16 in the
         // actual output (filter tensor).
-        kernelCount = 3;
+        kernelCount = 2;
       }
     }
   }
@@ -338,8 +296,9 @@ LogicalResult ConvGenerator::getBwdWeightKernelCount(OpBuilder &builder,
 }
 
 int ConvGenerator::getBwdDataKernelCount() const {
-  llvm::SmallVector<int64_t> gemmIds = backwardDataKernelIds(
-      config.strideDims, config.dilationDims, config.filterDims);
+  llvm::SmallVector<int64_t> gemmIds =
+      backwardDataKernelIds(config.strideDims, config.dilationDims,
+                            config.filterDims, config.usesV4R1);
   return static_cast<int>(gemmIds.size());
 }
 
@@ -392,7 +351,7 @@ LogicalResult ConvGenerator::needExtraPadBwdWeight(OpBuilder &builder,
   assert(dir == ConvOpType::BwdWeight &&
          "This method should only be called for wrw ops");
 
-  ConvolutionDims convDims = getConvolutionDims();
+  ConvolutionDims convDims = getConvolutionDims(&config);
   GemmSize gemmSize = GemmSize::fromConvolution(dir, convDims);
 
   needExtraPad = false;
@@ -533,6 +492,12 @@ LogicalResult ConvGenerator::parseConvConfig(OpBuilder &builder,
     }
   };
 
+  auto strToBool = [&argMap](const std::string &key, bool &setting) {
+    if (argMap.find(key) != argMap.end()) {
+      setting = (argMap[key] == "true");
+    }
+  };
+
   std::string arch;
   strToStr("arch", arch);
   RocmDeviceName splitter;
@@ -549,6 +514,10 @@ LogicalResult ConvGenerator::parseConvConfig(OpBuilder &builder,
   config.chipFeatures = splitter.getFeaturesForBackend();
   config.triple = splitter.getTriple().str();
 
+  bool usesV4R1Config = true;
+  strToBool("usesV4R1", usesV4R1Config);
+  config.usesV4R1 = usesV4R1Config;
+
   FailureOr<amdgpu::Chipset> maybeChipset = amdgpu::Chipset::parse(config.chip);
   if (failed(maybeChipset)) {
     emitError(UnknownLoc::get(builder.getContext()),
@@ -558,7 +527,6 @@ LogicalResult ConvGenerator::parseConvConfig(OpBuilder &builder,
 
   strToStr("perf_config", config.perfConfig);
   strToInt("num_cu", config.num_cu);
-  strToInt(rock::ReverseGridAttrAttr::getMnemonic().str(), config.reverseGrid);
 
   // conv settings
   auto const op = getConvOpTypeForName(argMap["operation"]);
@@ -572,12 +540,12 @@ LogicalResult ConvGenerator::parseConvConfig(OpBuilder &builder,
     if (type == "fp16")
       return std::string("f16");
     if (type == "fp8") {
-      if (maybeChipset->hasOcpFp8())
+      if (amdgpu::hasOcpFp8(maybeChipset.value()))
         return std::string("f8E4M3FN");
       return std::string("f8E4M3FNUZ");
     }
     if (type == "bf8") {
-      if (maybeChipset->hasOcpFp8())
+      if (amdgpu::hasOcpFp8(maybeChipset.value()))
         return std::string("f8E5M2");
       return std::string("f8E5M2FNUZ");
     }
@@ -770,28 +738,46 @@ void ConvGenerator::setPerfConfig(StringRef perfConfig) {
   config.perfConfig = perfConfig.str();
 }
 
-ConvolutionDims ConvGenerator::getConvolutionDims() const {
-  auto inDim = canonicalizeDims(config.inputDimension, config.inputLayout);
-  auto filDim = canonicalizeDims(config.filterDimension, config.filterLayout);
-  auto outDim = canonicalizeDims(config.outputDimension, config.outputLayout);
+ConvolutionDims ConvGenerator::getConvolutionDims(const Config *config) {
+  auto inDim = canonicalizeDims(config->inputDimension, config->inputLayout);
+  auto filDim = canonicalizeDims(config->filterDimension, config->filterLayout);
+  auto outDim = canonicalizeDims(config->outputDimension, config->outputLayout);
 
   SmallVector<int64_t> inDims;
-  for (size_t i = 0; i < config.inputLayout.size() - 3; i++)
+  for (size_t i = 0; i < config->inputLayout.size() - 3; i++)
     inDims.push_back(inDim[std::to_string(i)]);
   SmallVector<int64_t> filDims;
-  for (size_t i = 0; i < config.filterLayout.size() - 3; i++)
+  for (size_t i = 0; i < config->filterLayout.size() - 3; i++)
     filDims.push_back(filDim[std::to_string(i)]);
   SmallVector<int64_t> outDims;
-  for (size_t i = 0; i < config.outputLayout.size() - 3; i++)
+  for (size_t i = 0; i < config->outputLayout.size() - 3; i++)
     outDims.push_back(outDim[std::to_string(i)]);
 
   return ConvolutionDims(filDims, outDims, inDims, filDim["k"], filDim["c"],
                          inDim["n"], inDim["g"]);
 }
 
-LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
-                                           bool is_verifier,
-                                           bool ignoreTuning) {
+// Helper function to zero-initialize an argument of a FuncOp using the
+// prefill attribute
+static void zeroInitArg(OpBuilder &builder, func::FuncOp func,
+                        unsigned argIndex) {
+  auto argToPrefill = func.getArgument(argIndex);
+  auto attrName = rock::PrefillAttr::getMnemonic();
+  auto elementType = getElementTypeOrSelf(argToPrefill.getType());
+  Attribute zero;
+  if (isa<FloatType>(elementType)) {
+    zero = builder.getFloatAttr(elementType, 0.0);
+  } else {
+    assert(isa<IntegerType>(elementType) &&
+           "Unsupported element type for prefill attribute");
+    zero = builder.getIntegerAttr(elementType, 0);
+  }
+  func.setArgAttrs(argToPrefill.getArgNumber(),
+                   builder.getNamedAttr(attrName, zero));
+}
+
+LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int kernelId,
+                                           bool isVerifier, bool ignoreTuning) {
   OpBuilder builder(module.getContext());
 
   Type filterDataType = getFilterDataType(builder);
@@ -832,12 +818,13 @@ LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
     logicalFuncArgTypes = {filterArgType, inputArgType, outputArgType,
                            workspaceArgType};
   }
+
   SmallVector<Type, 3> physicalFuncArgTypes =
       llvm::map_to_vector(logicalFuncArgTypes, getFlattenedType);
   auto funcType = builder.getFunctionType(physicalFuncArgTypes, {});
 
   std::string kernelName = config.kernelBaseName;
-  if (is_verifier) {
+  if (isVerifier) {
     kernelName += "_ver";
   }
 
@@ -850,32 +837,34 @@ LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
   // Fix raw kernel ID in case it is less than 0.
   // The only case this could happen is to query the number of kernels needed
   // from MIIR API, where the kernel ID is not yet unknown.
-  if (rawKernelId < 0)
-    rawKernelId = 0;
+  if (kernelId < 0)
+    kernelId = 0;
 
   // Annotate kernel attribute to the FuncOp.
   StringAttr archStrAttr = builder.getStringAttr(config.arch);
   NamedAttribute archAttr = builder.getNamedAttr("mhal.arch", archStrAttr);
+  IntegerAttr numCUIntAttr =
+      builder.getIntegerAttr(builder.getI32Type(), getNumCU());
+  NamedAttribute numCUAttr = builder.getNamedAttr("num_cu", numCUIntAttr);
 
   SmallVector<NamedAttribute, 2> kernelAttrs = {
-      builder.getNamedAttr("kernel", builder.getI32IntegerAttr(rawKernelId)),
-      archAttr};
+      builder.getNamedAttr("kernel", builder.getI32IntegerAttr(kernelId)),
+      archAttr, numCUAttr};
 
   // Construct the FuncOp.
   func = func::FuncOp::create(builder.getUnknownLoc(), kernelName, funcType,
                               ArrayRef<NamedAttribute>(kernelAttrs));
-  // TODO[split-K]: split-K does not work with BwdWeight
-  if (!config.disableSplitKForTuning &&
-      config.operation.value() != ConvOpType::BwdWeight) {
+  if (!config.disableSplitKForTuning) {
     func->setAttr(rock::EnableSplitKForTuningAttr::getMnemonic(),
                   builder.getUnitAttr());
   }
-  if (config.reverseGrid) {
-    func->setAttr(rock::ReverseGridAttrAttr::getMnemonic(),
-                  builder.getUnitAttr());
+  if (config.scheduleVersion != 1) {
+    func->setAttr(rock::ScheduleVersionAttr::getMnemonic(),
+                  rock::ScheduleVersionAttr::get(builder.getContext(),
+                                                 config.scheduleVersion));
   }
   module.push_back(func);
-  if (!is_verifier)
+  if (!isVerifier)
     module->setAttr(archAttr.getName(), archAttr.getValue());
   if (func.getName() != kernelName) {
     return failure();
@@ -897,22 +886,7 @@ LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
   for (auto &key : config.outputLayout)
     outputLayoutSpec.push_back(builder.getStringAttr(StringRef(&key, 1) + "o"));
 
-  // Set kernel ID to  be the same as the raw kernel ID.
-  // For backward data convolution, additional processing is needed below.
-  int64_t kernelId = rawKernelId;
-
-  // Obtain kernel ID as used by backwards data kernels from the raw,
-  // 0-indexed kernel ID.
-  if (config.operation.value() == ConvOpType::BwdData) {
-    llvm::SmallVector<int64_t> kernelIds = backwardDataKernelIds(
-        config.strideDims, config.dilationDims, config.filterDims);
-    assert(kernelIds.size() > static_cast<size_t>(rawKernelId));
-    kernelId = kernelIds[rawKernelId];
-  }
-
   std::vector<NamedAttribute> attributes{
-      builder.getNamedAttr("arch", archStrAttr),
-
       builder.getNamedAttr(
           "filter_layout",
           builder.getArrayAttr(ArrayRef<Attribute>(filterLayoutSpec.begin(),
@@ -924,7 +898,6 @@ LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
           "output_layout",
           builder.getArrayAttr(ArrayRef<Attribute>(outputLayoutSpec.begin(),
                                                    outputLayoutSpec.end()))),
-      builder.getNamedAttr("numCU", builder.getI32IntegerAttr(getNumCU())),
   };
 
   // The backwards data kernel needs to know its kernel ID, as there are
@@ -933,6 +906,8 @@ LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
   if (config.operation.value() == ConvOpType::BwdData) {
     attributes.push_back(
         builder.getNamedAttr("kernelId", builder.getIndexAttr(kernelId)));
+    attributes.push_back(
+        builder.getNamedAttr("usesV4R1", builder.getBoolAttr(config.usesV4R1)));
   }
   // features
   GemmFeaturesAttr features =
@@ -978,52 +953,60 @@ LogicalResult ConvGenerator::genConvModule(ModuleOp &module, int rawKernelId,
                               logicalFuncArgTypes, args);
   switch (config.operation.value()) {
   case ConvOpType::Fwd: {
-    builder.create<ConvOp>(builder.getUnknownLoc(), ArrayRef<Type>{}, args,
-                           attributes);
+    ConvOp::create(builder, builder.getUnknownLoc(), ArrayRef<Type>{}, args,
+                   attributes);
   } break;
   case ConvOpType::BwdData: {
-    if (kernelId < 0) {
-      // zero init input tensor
-      builder.create<InitKernelOp>(
-          builder.getUnknownLoc(), /*resultType=*/TypeRange{},
-          func.getArgument(1), features, /*initValueAttr=*/nullptr,
-          /*blockSize=*/nullptr, /*gridSize=*/nullptr,
-          /*elemsPerThread=*/nullptr);
-    } else {
-      builder.create<ConvBwdDataOp>(builder.getUnknownLoc(), ArrayRef<Type>{},
-                                    args, attributes);
+    if (!rock::isEveryElementWrittenBwdData(
+            config.strideDims, config.dilationDims, config.filterDims)) {
+      // For all backwards data convolution ops that don't write to every pixel,
+      // we want to zeroinitialize the buffer in the second argument
+      // (input tensor)
+      // TODO: This is okay for right now since we are not doing any fusions.
+      // When we do handle fusions in the future there is no guarantee that
+      // arg 1 is going to be the input tensor.
+      zeroInitArg(builder, func, 1);
     }
+    builder.create<ConvBwdDataOp>(builder.getUnknownLoc(), ArrayRef<Type>{},
+                                  args, attributes);
   } break;
   case ConvOpType::BwdWeight: {
     int kernelCount = 0;
     if (failed(getBwdWeightKernelCount(builder, kernelCount))) {
       return failure();
     }
-    bool hasUtilities = (kernelCount > 1);
-    if (hasUtilities && kernelId == 0) {
-      // If there is a workspace, zero-init it, otherwise fill the filter
-      // tensor
-      builder.create<InitKernelOp>(builder.getUnknownLoc(),
-                                   /*resultType=*/TypeRange{},
-                                   func.getArgument(hasWorkspace ? 3 : 0),
-                                   features, /*initValueAttr=*/nullptr,
-                                   /*blockSize=*/nullptr, /*gridSize=*/nullptr,
-                                   /*elemsPerThread=*/nullptr);
-    } else if (hasUtilities && kernelId == 2) {
+
+    bool needsZeroInit = false;
+    bool needExtraPad = false;
+    if (rock::isAccel(config.features) &&
+        succeeded(needExtraPadBwdWeight(builder, needExtraPad))) {
+      if (!needExtraPad) {
+        auto dataType = getInputDataType(builder);
+        needsZeroInit = dataType.isF32() || dataType.isF16();
+      }
+    }
+
+    if (kernelId == 1) {
       // Workspace -> filter tensor
-      builder.create<ConvertingCopyKernelOp>(
-          builder.getUnknownLoc(), /*resultType=*/TypeRange{},
-          func.getArgument(3), func.getArgument(0), features,
+      ConvertingCopyKernelOp::create(
+          builder, builder.getUnknownLoc(), /*resultType=*/TypeRange{},
+          func.getArgument(3), func.getArgument(0),
           /*blockSize=*/nullptr, /*gridSize=*/nullptr,
           /*elemsPerThread=*/nullptr);
     } else {
-      builder.create<ConvBwdWeightOp>(builder.getUnknownLoc(), ArrayRef<Type>{},
-                                      args, attributes);
+      // TODO: This is okay for right now since we are not doing any fusions.
+      // When we do handle fusions in the future there is no guarantee that
+      // these specific args are going to be the input tensor.
+      if (needsZeroInit) {
+        zeroInitArg(builder, func, hasWorkspace ? 3 : 0);
+      }
+      ConvBwdWeightOp::create(builder, builder.getUnknownLoc(),
+                              ArrayRef<Type>{}, args, attributes);
     }
   } break;
   }
 
-  builder.create<func::ReturnOp>(builder.getUnknownLoc(), ValueRange{});
+  func::ReturnOp::create(builder, builder.getUnknownLoc(), ValueRange{});
   return success();
 }
 
